@@ -27,6 +27,7 @@ from ironic_lib import disk_utils
 from ironic_lib import utils as ironic_utils
 from oslo_concurrency import processutils
 from oslo_serialization import base64
+import requests
 import testtools
 
 from ironic_python_agent import errors
@@ -146,198 +147,124 @@ class GetAgentParamsTestCase(ironic_agent_base.IronicAgentTest):
         vmedia_device_returned = utils._get_vmedia_device()
         self.assertEqual('sdc', vmedia_device_returned)
 
-    @mock.patch.object(shutil, 'rmtree', autospec=True)
-    @mock.patch.object(tempfile, 'mkdtemp', autospec=True)
-    @mock.patch.object(utils, '_read_params_from_file', autospec=True)
-    @mock.patch.object(os.path, 'exists', autospec=True)
-    @mock.patch.object(os, 'mkdir', autospec=True)
     @mock.patch.object(utils, 'execute', autospec=True)
-    def test__get_vmedia_params_by_label_lower_case(
-            self, execute_mock, mkdir_mock, exists_mock, read_params_mock,
-            mkdtemp_mock, rmtree_mock):
-        mkdtemp_mock.return_value = "/tempdir"
+    def test__find_vmedia_device_by_labels_handles_exec_error(self,
+                                                              execute_mock):
+        execute_mock.side_effect = processutils.ProcessExecutionError
+        self.assertIsNone(utils._find_vmedia_device_by_labels(['l1', 'l2']))
+        execute_mock.assert_called_once_with('lsblk', '-p', '-P',
+                                             '-oKNAME,LABEL')
 
-        null_output = ["", ""]
+    @mock.patch.object(utils, 'execute', autospec=True)
+    def test__find_vmedia_device_by_labels(self, execute_mock):
+        # NOTE(TheJulia): Case is intentionally mixed here to ensure
+        # proper matching occurs
+        disk_list = ('KNAME="/dev/sda" LABEL=""\n'
+                     'KNAME="/dev/sda2" LABEL="Meow"\n'
+                     'KNAME="/dev/sda3" LABEL="Recovery HD"\n'
+                     'KNAME="/dev/sda1" LABEL="EFI"\n'
+                     'KNAME="/dev/sdb" LABEL=""\n'
+                     'KNAME="/dev/sdb1" LABEL=""\n'
+                     'KNAME="/dev/sdb2" LABEL=""\n'
+                     'KNAME="/dev/sdc" LABEL="meow"\n')
+        invalid_disk = ('KNAME="sda1" SIZE="1610612736" TYPE="part" TRAN=""\n'
+                        'KNAME="sda" SIZE="1610612736" TYPE="disk" '
+                        'TRAN="sata"\n')
+        valid_disk = ('KNAME="sdc" SIZE="1610612736" TYPE="disk" TRAN="usb"\n')
+        execute_mock.side_effect = [
+            (disk_list, ''),
+            (invalid_disk, ''),
+            (valid_disk, ''),
+        ]
+        self.assertEqual('/dev/sdc',
+                         utils._find_vmedia_device_by_labels(['cat', 'meOw']))
+        execute_mock.assert_has_calls([
+            mock.call('lsblk', '-p', '-P', '-oKNAME,LABEL'),
+            mock.call('lsblk', '-n', '-s', '-P', '-b',
+                      '-oKNAME,TRAN,TYPE,SIZE', '/dev/sda2'),
+            mock.call('lsblk', '-n', '-s', '-P', '-b',
+                      '-oKNAME,TRAN,TYPE,SIZE', '/dev/sdc'),
+        ])
+
+    @mock.patch.object(utils, 'execute', autospec=True)
+    def test__find_vmedia_device_by_labels_not_found(self, execute_mock):
+        disk_list = ('KNAME="/dev/sdb" LABEL="evil"\n'
+                     'KNAME="/dev/sdb1" LABEL="banana"\n'
+                     'KNAME="/dev/sdb2" LABEL=""\n')
+        execute_mock.return_value = (disk_list, '')
+        self.assertIsNone(utils._find_vmedia_device_by_labels(['l1', 'l2']))
+        execute_mock.assert_called_once_with('lsblk', '-p', '-P',
+                                             '-oKNAME,LABEL')
+
+    @mock.patch.object(utils, '_check_vmedia_device', autospec=True)
+    @mock.patch.object(utils, '_find_vmedia_device_by_labels', autospec=True)
+    @mock.patch.object(utils, '_read_params_from_file', autospec=True)
+    @mock.patch.object(ironic_utils, 'mounted', autospec=True)
+    def test__get_vmedia_params(self, mount_mock, read_params_mock, find_mock,
+                                check_vmedia_mock):
+        check_vmedia_mock.return_value = True
+        find_mock.return_value = '/dev/fake'
+        mount_mock.return_value.__enter__.return_value = '/tempdir'
         expected_params = {'a': 'b'}
         read_params_mock.return_value = expected_params
-        exists_mock.side_effect = [True, False]
-        execute_mock.side_effect = [null_output, null_output]
 
         returned_params = utils._get_vmedia_params()
 
-        execute_mock.assert_any_call('mount', "/dev/disk/by-label/ir-vfd-dev",
-                                     "/tempdir")
+        mount_mock.assert_called_once_with('/dev/fake')
         read_params_mock.assert_called_once_with("/tempdir/parameters.txt")
-        exists_mock.assert_called_once_with("/dev/disk/by-label/ir-vfd-dev")
-        execute_mock.assert_any_call('umount', "/tempdir")
         self.assertEqual(expected_params, returned_params)
-        mkdtemp_mock.assert_called_once_with()
-        rmtree_mock.assert_called_once_with("/tempdir")
 
-    @mock.patch.object(shutil, 'rmtree', autospec=True)
-    @mock.patch.object(tempfile, 'mkdtemp', autospec=True)
-    @mock.patch.object(utils, '_read_params_from_file', autospec=True)
-    @mock.patch.object(os.path, 'exists', autospec=True)
-    @mock.patch.object(os, 'mkdir', autospec=True)
-    @mock.patch.object(utils, 'execute', autospec=True)
-    def test__get_vmedia_params_by_label_upper_case(
-            self, execute_mock, mkdir_mock, exists_mock, read_params_mock,
-            mkdtemp_mock, rmtree_mock):
-        mkdtemp_mock.return_value = "/tempdir"
-
-        null_output = ["", ""]
-        expected_params = {'a': 'b'}
-        read_params_mock.return_value = expected_params
-        exists_mock.side_effect = [False, True]
-        execute_mock.side_effect = [null_output, null_output]
-
-        returned_params = utils._get_vmedia_params()
-
-        execute_mock.assert_any_call('mount', "/dev/disk/by-label/IR-VFD-DEV",
-                                     "/tempdir")
-        read_params_mock.assert_called_once_with("/tempdir/parameters.txt")
-        exists_mock.assert_has_calls(
-            [mock.call("/dev/disk/by-label/ir-vfd-dev"),
-             mock.call("/dev/disk/by-label/IR-VFD-DEV")])
-        execute_mock.assert_any_call('umount', "/tempdir")
-        self.assertEqual(expected_params, returned_params)
-        mkdtemp_mock.assert_called_once_with()
-        rmtree_mock.assert_called_once_with("/tempdir")
-
-    @mock.patch.object(shutil, 'rmtree', autospec=True)
-    @mock.patch.object(tempfile, 'mkdtemp', autospec=True)
+    @mock.patch.object(utils, '_check_vmedia_device', autospec=True)
+    @mock.patch.object(utils, '_find_vmedia_device_by_labels', autospec=True)
     @mock.patch.object(utils, '_get_vmedia_device', autospec=True)
     @mock.patch.object(utils, '_read_params_from_file', autospec=True)
-    @mock.patch.object(os.path, 'exists', autospec=True)
-    @mock.patch.object(os, 'mkdir', autospec=True)
-    @mock.patch.object(utils, 'execute', autospec=True)
-    def test__get_vmedia_params_by_device(self, execute_mock, mkdir_mock,
-                                          exists_mock, read_params_mock,
-                                          get_device_mock, mkdtemp_mock,
-                                          rmtree_mock):
-        mkdtemp_mock.return_value = "/tempdir"
-
-        null_output = ["", ""]
+    @mock.patch.object(ironic_utils, 'mounted', autospec=True)
+    def test__get_vmedia_params_by_device(self, mount_mock, read_params_mock,
+                                          get_device_mock, find_mock,
+                                          check_vmedia_mock):
+        check_vmedia_mock.return_value = True
+        find_mock.return_value = None
+        mount_mock.return_value.__enter__.return_value = '/tempdir'
         expected_params = {'a': 'b'}
         read_params_mock.return_value = expected_params
-        exists_mock.side_effect = [False, False]
-        execute_mock.side_effect = [null_output, null_output]
         get_device_mock.return_value = "sda"
 
         returned_params = utils._get_vmedia_params()
 
-        exists_mock.assert_has_calls(
-            [mock.call("/dev/disk/by-label/ir-vfd-dev"),
-             mock.call("/dev/disk/by-label/IR-VFD-DEV")])
-        execute_mock.assert_any_call('mount', "/dev/sda",
-                                     "/tempdir")
+        mount_mock.assert_called_once_with('/dev/sda')
         read_params_mock.assert_called_once_with("/tempdir/parameters.txt")
-        execute_mock.assert_any_call('umount', "/tempdir")
         self.assertEqual(expected_params, returned_params)
-        mkdtemp_mock.assert_called_once_with()
-        rmtree_mock.assert_called_once_with("/tempdir")
+        check_vmedia_mock.assert_called_with('/dev/sda')
 
+    @mock.patch.object(utils, '_check_vmedia_device', autospec=True)
+    @mock.patch.object(utils, '_find_vmedia_device_by_labels', autospec=True)
     @mock.patch.object(utils, '_get_vmedia_device', autospec=True)
-    @mock.patch.object(os.path, 'exists', autospec=True)
-    def test__get_vmedia_params_cannot_find_dev(self, exists_mock,
-                                                get_device_mock):
+    @mock.patch.object(utils, '_read_params_from_file', autospec=True)
+    @mock.patch.object(ironic_utils, 'mounted', autospec=True)
+    def test__get_vmedia_params_by_device_device_invalid(
+            self, mount_mock, read_params_mock,
+            get_device_mock, find_mock,
+            check_vmedia_mock):
+        check_vmedia_mock.return_value = False
+        find_mock.return_value = None
+        expected_params = {}
+        read_params_mock.return_value = expected_params
+        get_device_mock.return_value = "sda"
+
+        returned_params = utils._get_vmedia_params()
+
+        mount_mock.assert_not_called()
+        read_params_mock.assert_not_called
+        self.assertEqual(expected_params, returned_params)
+        check_vmedia_mock.assert_called_with('/dev/sda')
+
+    @mock.patch.object(utils, '_find_vmedia_device_by_labels', autospec=True)
+    @mock.patch.object(utils, '_get_vmedia_device', autospec=True)
+    def test__get_vmedia_params_cannot_find_dev(self, get_device_mock,
+                                                find_mock):
+        find_mock.return_value = None
         get_device_mock.return_value = None
-        exists_mock.return_value = False
-        self.assertRaises(errors.VirtualMediaBootError,
-                          utils._get_vmedia_params)
-
-    @mock.patch.object(shutil, 'rmtree', autospec=True)
-    @mock.patch.object(tempfile, 'mkdtemp', autospec=True)
-    @mock.patch.object(utils, '_get_vmedia_device', autospec=True)
-    @mock.patch.object(utils, '_read_params_from_file', autospec=True)
-    @mock.patch.object(os.path, 'exists', autospec=True)
-    @mock.patch.object(os, 'mkdir', autospec=True)
-    @mock.patch.object(utils, 'execute', autospec=True)
-    def test__get_vmedia_params_mount_fails(self, execute_mock,
-                                            mkdir_mock, exists_mock,
-                                            read_params_mock,
-                                            get_device_mock, mkdtemp_mock,
-                                            rmtree_mock):
-        mkdtemp_mock.return_value = "/tempdir"
-
-        expected_params = {'a': 'b'}
-        exists_mock.return_value = True
-        read_params_mock.return_value = expected_params
-        get_device_mock.return_value = "sda"
-
-        execute_mock.side_effect = processutils.ProcessExecutionError()
-
-        self.assertRaises(errors.VirtualMediaBootError,
-                          utils._get_vmedia_params)
-
-        execute_mock.assert_any_call('mount', "/dev/disk/by-label/ir-vfd-dev",
-                                     "/tempdir")
-        mkdtemp_mock.assert_called_once_with()
-        rmtree_mock.assert_called_once_with("/tempdir")
-
-    @mock.patch.object(shutil, 'rmtree', autospec=True)
-    @mock.patch.object(tempfile, 'mkdtemp', autospec=True)
-    @mock.patch.object(utils, '_get_vmedia_device', autospec=True)
-    @mock.patch.object(utils, '_read_params_from_file', autospec=True)
-    @mock.patch.object(os.path, 'exists', autospec=True)
-    @mock.patch.object(os, 'mkdir', autospec=True)
-    @mock.patch.object(utils, 'execute', autospec=True)
-    def test__get_vmedia_params_umount_fails(self, execute_mock, mkdir_mock,
-                                             exists_mock, read_params_mock,
-                                             get_device_mock, mkdtemp_mock,
-                                             rmtree_mock):
-        mkdtemp_mock.return_value = "/tempdir"
-
-        null_output = ["", ""]
-        expected_params = {'a': 'b'}
-        exists_mock.return_value = True
-        read_params_mock.return_value = expected_params
-        get_device_mock.return_value = "sda"
-
-        execute_mock.side_effect = [null_output,
-                                    processutils.ProcessExecutionError()]
-
-        returned_params = utils._get_vmedia_params()
-
-        execute_mock.assert_any_call('mount', "/dev/disk/by-label/ir-vfd-dev",
-                                     "/tempdir")
-        read_params_mock.assert_called_once_with("/tempdir/parameters.txt")
-        execute_mock.assert_any_call('umount', "/tempdir")
-        self.assertEqual(expected_params, returned_params)
-        mkdtemp_mock.assert_called_once_with()
-        rmtree_mock.assert_called_once_with("/tempdir")
-
-    @mock.patch.object(shutil, 'rmtree', autospec=True)
-    @mock.patch.object(tempfile, 'mkdtemp', autospec=True)
-    @mock.patch.object(utils, '_get_vmedia_device', autospec=True)
-    @mock.patch.object(utils, '_read_params_from_file', autospec=True)
-    @mock.patch.object(os.path, 'exists', autospec=True)
-    @mock.patch.object(os, 'mkdir', autospec=True)
-    @mock.patch.object(utils, 'execute', autospec=True)
-    def test__get_vmedia_params_rmtree_fails(self, execute_mock, mkdir_mock,
-                                             exists_mock, read_params_mock,
-                                             get_device_mock, mkdtemp_mock,
-                                             rmtree_mock):
-        mkdtemp_mock.return_value = "/tempdir"
-        rmtree_mock.side_effect = Exception
-
-        null_output = ["", ""]
-        expected_params = {'a': 'b'}
-        exists_mock.return_value = True
-        read_params_mock.return_value = expected_params
-        get_device_mock.return_value = "sda"
-
-        execute_mock.return_value = null_output
-
-        returned_params = utils._get_vmedia_params()
-
-        execute_mock.assert_any_call('mount', "/dev/disk/by-label/ir-vfd-dev",
-                                     "/tempdir")
-        read_params_mock.assert_called_once_with("/tempdir/parameters.txt")
-        execute_mock.assert_any_call('umount', "/tempdir")
-        self.assertEqual(expected_params, returned_params)
-        mkdtemp_mock.assert_called_once_with()
-        rmtree_mock.assert_called_once_with("/tempdir")
+        self.assertEqual({}, utils._get_vmedia_params())
 
 
 class TestFailures(testtools.TestCase):
@@ -929,8 +856,27 @@ class TestClockSyncUtils(ironic_agent_base.IronicAgentTest):
         mock_time_method.return_value = 'chronyd'
         utils.sync_clock()
         mock_execute.assert_has_calls([
-            mock.call('chronyc', 'shutdown', check_exit_code=[0, 1]),
-            mock.call("chronyd -q 'server 192.168.1.1 iburst'", shell=True),
+            mock.call('chronyd', check_exit_code=[0, 1]),
+            mock.call('chronyc', 'add', 'server', '192.168.1.1'),
+            mock.call('chronyc', 'makestep'),
+        ])
+
+    @mock.patch.object(utils, 'determine_time_method', autospec=True)
+    def test_sync_clock_chrony_already_present(self, mock_time_method,
+                                               mock_execute):
+        self.config(ntp_server='192.168.1.1')
+        mock_time_method.return_value = 'chronyd'
+        mock_execute.side_effect = [
+            ('', ''),
+            processutils.ProcessExecutionError(
+                stderr='Source already present'),
+            ('', ''),
+        ]
+        utils.sync_clock()
+        mock_execute.assert_has_calls([
+            mock.call('chronyd', check_exit_code=[0, 1]),
+            mock.call('chronyc', 'add', 'server', '192.168.1.1'),
+            mock.call('chronyc', 'makestep'),
         ])
 
     @mock.patch.object(utils, 'determine_time_method', autospec=True)
@@ -943,8 +889,12 @@ class TestClockSyncUtils(ironic_agent_base.IronicAgentTest):
             processutils.ProcessExecutionError(stderr='time verboten'),
         ]
         self.assertRaisesRegex(errors.CommandExecutionError,
-                               'Failed to sync time using chrony to ntp '
-                               'server:', utils.sync_clock)
+                               'Error occured adding ntp',
+                               utils.sync_clock)
+        mock_execute.assert_has_calls([
+            mock.call('chronyd', check_exit_code=[0, 1]),
+            mock.call('chronyc', 'add', 'server', '192.168.1.1'),
+        ])
 
     @mock.patch.object(utils, 'determine_time_method', autospec=True)
     def test_sync_clock_none(self, mock_time_method, mock_execute):
@@ -1002,3 +952,248 @@ class TestGetEfiPart(testtools.TestCase):
             {'number': '14', 'flags': 'bios_grub'},
         ]
         self.assertIsNone(utils.get_efi_part_on_device('/dev/sda'))
+
+
+@mock.patch.object(utils, '_booted_from_vmedia', autospec=True)
+@mock.patch.object(utils, '_check_vmedia_device', autospec=True)
+@mock.patch.object(utils, '_find_vmedia_device_by_labels', autospec=True)
+@mock.patch.object(shutil, 'copy', autospec=True)
+@mock.patch.object(ironic_utils, 'mounted', autospec=True)
+@mock.patch.object(utils, 'execute', autospec=True)
+class TestCopyConfigFromVmedia(testtools.TestCase):
+
+    def test_vmedia_found_not_booted_from_vmedia(
+            self, mock_execute, mock_mount, mock_copy,
+            mock_find_device, mock_check_vmedia, mock_booted_from_vmedia):
+        mock_booted_from_vmedia.return_value = False
+        mock_find_device.return_value = '/dev/fake'
+        utils.copy_config_from_vmedia()
+        mock_mount.assert_not_called()
+        mock_execute.assert_not_called()
+        mock_copy.assert_not_called()
+        mock_check_vmedia.assert_not_called()
+        self.assertTrue(mock_booted_from_vmedia.called)
+
+    def test_no_vmedia(
+            self, mock_execute, mock_mount, mock_copy,
+            mock_find_device, mock_check_vmedia, mock_booted_from_vmedia):
+        mock_booted_from_vmedia.return_value = True
+        mock_find_device.return_value = None
+        utils.copy_config_from_vmedia()
+        mock_mount.assert_not_called()
+        mock_execute.assert_not_called()
+        mock_copy.assert_not_called()
+        mock_check_vmedia.assert_not_called()
+        self.assertFalse(mock_booted_from_vmedia.called)
+
+    def test_no_files(
+            self, mock_execute, mock_mount, mock_copy,
+            mock_find_device, mock_check_vmedia, mock_booted_from_vmedia):
+        mock_booted_from_vmedia.return_value = True
+        temp_path = tempfile.mkdtemp()
+        self.addCleanup(lambda: shutil.rmtree(temp_path))
+
+        mock_execute.side_effect = processutils.ProcessExecutionError
+        mock_find_device.return_value = '/dev/something'
+        mock_mount.return_value.__enter__.return_value = temp_path
+        utils.copy_config_from_vmedia()
+        mock_mount.assert_called_once_with('/dev/something')
+        mock_execute.assert_called_once_with('findmnt', '-n', '-oTARGET',
+                                             '/dev/something')
+        mock_copy.assert_not_called()
+        self.assertTrue(mock_booted_from_vmedia.called)
+
+    def test_mounted_no_files(
+            self, mock_execute, mock_mount, mock_copy,
+            mock_find_device, mock_check_vmedia, mock_booted_from_vmedia):
+
+        mock_booted_from_vmedia.return_value = True
+        mock_execute.return_value = '/some/path', ''
+        mock_find_device.return_value = '/dev/something'
+        utils.copy_config_from_vmedia()
+        mock_execute.assert_called_once_with(
+            'findmnt', '-n', '-oTARGET', '/dev/something')
+        mock_copy.assert_not_called()
+        mock_mount.assert_not_called()
+        self.assertTrue(mock_booted_from_vmedia.called)
+
+    @mock.patch.object(os, 'makedirs', autospec=True)
+    def test_copy(
+            self, mock_makedirs, mock_execute, mock_mount, mock_copy,
+            mock_find_device, mock_check_vmedia, mock_booted_from_vmedia):
+
+        mock_booted_from_vmedia.return_value = True
+        mock_find_device.return_value = '/dev/something'
+        mock_execute.side_effect = processutils.ProcessExecutionError("")
+        path = tempfile.mkdtemp()
+        self.addCleanup(lambda: shutil.rmtree(path))
+
+        def _fake_mount(dev):
+            self.assertEqual('/dev/something', dev)
+            # NOTE(dtantsur): makedirs is mocked
+            os.mkdir(os.path.join(path, 'etc'))
+            os.mkdir(os.path.join(path, 'etc', 'ironic-python-agent'))
+            os.mkdir(os.path.join(path, 'etc', 'ironic-python-agent.d'))
+            with open(os.path.join(path, 'not copied'), 'wt') as fp:
+                fp.write('not copied')
+            with open(os.path.join(path, 'etc', 'ironic-python-agent',
+                                   'ironic.crt'), 'wt') as fp:
+                fp.write('I am a cert')
+            with open(os.path.join(path, 'etc', 'ironic-python-agent.d',
+                                   'ironic.conf'), 'wt') as fp:
+                fp.write('I am a config')
+            return mock.MagicMock(**{'__enter__.return_value': path})
+
+        mock_find_device.return_value = '/dev/something'
+        mock_mount.side_effect = _fake_mount
+
+        utils.copy_config_from_vmedia()
+
+        mock_makedirs.assert_has_calls([
+            mock.call('/etc/ironic-python-agent', exist_ok=True),
+            mock.call('/etc/ironic-python-agent.d', exist_ok=True),
+        ], any_order=True)
+        mock_mount.assert_called_once_with('/dev/something')
+        mock_copy.assert_has_calls([
+            mock.call(mock.ANY, '/etc/ironic-python-agent/ironic.crt'),
+            mock.call(mock.ANY, '/etc/ironic-python-agent.d/ironic.conf'),
+        ], any_order=True)
+        self.assertTrue(mock_booted_from_vmedia.called)
+
+    @mock.patch.object(os, 'makedirs', autospec=True)
+    def test_copy_mounted(
+            self, mock_makedirs, mock_execute, mock_mount,
+            mock_copy, mock_find_device, mock_check_vmedia,
+            mock_booted_from_vmedia):
+        mock_booted_from_vmedia.return_value = True
+        mock_find_device.return_value = '/dev/something'
+        path = tempfile.mkdtemp()
+        self.addCleanup(lambda: shutil.rmtree(path))
+
+        # NOTE(dtantsur): makedirs is mocked
+        os.mkdir(os.path.join(path, 'etc'))
+        os.mkdir(os.path.join(path, 'etc', 'ironic-python-agent'))
+        os.mkdir(os.path.join(path, 'etc', 'ironic-python-agent.d'))
+        with open(os.path.join(path, 'not copied'), 'wt') as fp:
+            fp.write('not copied')
+        with open(os.path.join(path, 'etc', 'ironic-python-agent',
+                               'ironic.crt'), 'wt') as fp:
+            fp.write('I am a cert')
+        with open(os.path.join(path, 'etc', 'ironic-python-agent.d',
+                               'ironic.conf'), 'wt') as fp:
+            fp.write('I am a config')
+
+        mock_execute.return_value = path, ''
+        mock_find_device.return_value = '/dev/something'
+
+        utils.copy_config_from_vmedia()
+
+        mock_makedirs.assert_has_calls([
+            mock.call('/etc/ironic-python-agent', exist_ok=True),
+            mock.call('/etc/ironic-python-agent.d', exist_ok=True),
+        ], any_order=True)
+        mock_execute.assert_called_once_with(
+            'findmnt', '-n', '-oTARGET', '/dev/something')
+        mock_copy.assert_has_calls([
+            mock.call(mock.ANY, '/etc/ironic-python-agent/ironic.crt'),
+            mock.call(mock.ANY, '/etc/ironic-python-agent.d/ironic.conf'),
+        ], any_order=True)
+        mock_mount.assert_not_called()
+        self.assertTrue(mock_booted_from_vmedia.called)
+
+
+@mock.patch.object(requests, 'get', autospec=True)
+class TestStreamingClient(ironic_agent_base.IronicAgentTest):
+
+    def test_ok(self, mock_get):
+        client = utils.StreamingClient()
+        self.assertTrue(client.verify)
+        self.assertIsNone(client.cert)
+
+        with client("http://url") as result:
+            response = mock_get.return_value.__enter__.return_value
+            self.assertIs(result, response.iter_content.return_value)
+
+        mock_get.assert_called_once_with("http://url", verify=True, cert=None,
+                                         stream=True, timeout=60)
+        response.iter_content.assert_called_once_with(1024 * 1024)
+
+    def test_retries(self, mock_get):
+        self.config(image_download_connection_retries=1,
+                    image_download_connection_retry_interval=1)
+        mock_get.side_effect = requests.ConnectionError
+
+        client = utils.StreamingClient()
+        self.assertRaises(errors.CommandExecutionError,
+                          client("http://url").__enter__)
+
+        mock_get.assert_called_with("http://url", verify=True, cert=None,
+                                    stream=True, timeout=60)
+        self.assertEqual(2, mock_get.call_count)
+
+
+class TestCheckVirtualMedia(ironic_agent_base.IronicAgentTest):
+
+    @mock.patch.object(utils, 'execute', autospec=True)
+    def test_check_vmedia_device(self, mock_execute):
+        lsblk = 'KNAME="sdh" SIZE="1610612736" TYPE="disk" TRAN="usb"\n'
+        mock_execute.return_value = (lsblk, '')
+        self.assertTrue(utils._check_vmedia_device('/dev/sdh'))
+        mock_execute.assert_called_with('lsblk', '-n', '-s', '-P', '-b',
+                                        '-oKNAME,TRAN,TYPE,SIZE',
+                                        '/dev/sdh')
+
+    @mock.patch.object(utils, 'execute', autospec=True)
+    def test_check_vmedia_device_rom(self, mock_execute):
+        lsblk = 'KNAME="sr0" SIZE="1610612736" TYPE="rom" TRAN="usb"\n'
+        mock_execute.return_value = (lsblk, '')
+        self.assertTrue(utils._check_vmedia_device('/dev/sr0'))
+        mock_execute.assert_called_with('lsblk', '-n', '-s', '-P', '-b',
+                                        '-oKNAME,TRAN,TYPE,SIZE',
+                                        '/dev/sr0')
+
+    @mock.patch.object(utils, 'execute', autospec=True)
+    def test_check_vmedia_device_too_large(self, mock_execute):
+        lsblk = 'KNAME="sdh" SIZE="1610612736000" TYPE="disk" TRAN="usb"\n'
+        mock_execute.return_value = (lsblk, '')
+        self.assertFalse(utils._check_vmedia_device('/dev/sdh'))
+        mock_execute.assert_called_with('lsblk', '-n', '-s', '-P', '-b',
+                                        '-oKNAME,TRAN,TYPE,SIZE',
+                                        '/dev/sdh')
+
+    @mock.patch.object(utils, 'execute', autospec=True)
+    def test_check_vmedia_device_part(self, mock_execute):
+        lsblk = ('KNAME="sdh1" SIZE="1610612736" TYPE="part" TRAN=""\n'
+                 'KNAME="sdh" SIZE="1610612736" TYPE="disk" TRAN="sata"\n')
+        mock_execute.return_value = (lsblk, '')
+        self.assertFalse(utils._check_vmedia_device('/dev/sdh1'))
+        mock_execute.assert_called_with('lsblk', '-n', '-s', '-P', '-b',
+                                        '-oKNAME,TRAN,TYPE,SIZE',
+                                        '/dev/sdh1')
+
+    @mock.patch.object(utils, 'execute', autospec=True)
+    def test_check_vmedia_device_other(self, mock_execute):
+        lsblk = 'KNAME="sdh" SIZE="1610612736" TYPE="other" TRAN="usb"\n'
+        mock_execute.return_value = (lsblk, '')
+        self.assertFalse(utils._check_vmedia_device('/dev/sdh'))
+        mock_execute.assert_called_with('lsblk', '-n', '-s', '-P', '-b',
+                                        '-oKNAME,TRAN,TYPE,SIZE',
+                                        '/dev/sdh')
+
+    @mock.patch.object(utils, 'execute', autospec=True)
+    def test_check_vmedia_device_sata(self, mock_execute):
+        lsblk = 'KNAME="sdh" SIZE="1610612736" TYPE="disk" TRAN="sata"\n'
+        mock_execute.return_value = (lsblk, '')
+        self.assertFalse(utils._check_vmedia_device('/dev/sdh'))
+        mock_execute.assert_called_with('lsblk', '-n', '-s', '-P', '-b',
+                                        '-oKNAME,TRAN,TYPE,SIZE',
+                                        '/dev/sdh')
+
+    @mock.patch.object(utils, 'execute', autospec=True)
+    def test_check_vmedia_device_scsi(self, mock_execute):
+        lsblk = 'KNAME="sdh" SIZE="1610612736" TYPE="other" TRAN="scsi"\n'
+        mock_execute.return_value = (lsblk, '')
+        self.assertFalse(utils._check_vmedia_device('/dev/sdh'))
+        mock_execute.assert_called_with('lsblk', '-n', '-s', '-P', '-b',
+                                        '-oKNAME,TRAN,TYPE,SIZE',
+                                        '/dev/sdh')
